<?php

namespace SuperRhino\Utils\Cache;

class MemoryCacheAdapter implements MemoryCacheAdapterInterface {

    private $cache = [];

    /**
     * @var JsonPacker
     */
    protected $packer;

    public function __construct() {
        $this->packer = new JsonPacker();
    }

    public function get(string $key, mixed $default = null): mixed {
        if ($this->has($key)) {
            return $this->packer->unpack($this->cache[$key]);
        }

        return $default;
    }

    public function getCallback($key, callable $callback, $ttl = null, ?float $beta = null) {
        if ($this->has($key)) {
            return $this->packer->unpack($this->cache[$key]);
        }
        $data = $callback();
        if ($data !== null) {
            $this->set($key, $data);
            return $data;
        }
        return null;
    }

    public function fetch(string $key, $default = null): array {
        if ($this->has($key)) {
            return [true, $this->packer->unpack($this->cache[$key])];
        }

        return [false, $default];
    }

    public function set(string $key, mixed $value, null|int|\DateInterval $ttl = null): bool {
        $this->cache[$key] = $this->packer->pack($value);
        return true;
    }

    public function delete(string $key): bool {
        unset($this->cache[$key]);
        return true;
    }

    public function clear(): bool {
        $this->cache = [];
        return true;
    }

    public function getMultiple($keys, $default = null): iterable {
        $results = [];
        foreach ($keys as $key) {
            $results[$key] = $this->get($key, $default);
        }
        return $results;
    }

    public function setMultiple($values, $ttl = null): bool {
        foreach ($values as $key => $value) {
            $this->set($key, $value);
        }
        return true;
    }

    public function deleteMultiple($keys): bool {
        foreach ($keys as $key) {
            $this->delete($key);
        }
        return true;
    }

    public function has($key): bool {
        return array_key_exists($key, $this->cache);
    }

    public function clearPrefix(string $prefix): bool {
        // TODO: Implement clearPrefix() method.
        return true;
    }

    public function addKey(string $collector, string $key): bool {
        $data = $this->has($collector) ? $this->cache[$collector] : [];
        $data[] = $key;
        $this->cache[$collector] = $data;
        return true;
    }

    public function keys(string $collector): array {
        return $this->has($collector) ? $this->cache[$collector] : [];
    }

    public function delKey(string $collector, ...$key): bool {
        $result = [];
        $data = $this->has($collector) ? $this->cache[$collector] : [];
        foreach ($data as $item) {
            if (!in_array($item, $key)) {
                $result[] = $item;
            }
        }
        $this->cache[$collector] = $result;
        return true;
    }

}