<?php

namespace SuperRhino\Utils\Redis;

use DateInterval;
use SuperRhino\Utils\Time\InteractsWithTime;

/**
 * @deprecated 兼容老系统缓存, 已弃用, 新代码使用Cache:: 方式
 */
class CustomCache {

    use InteractsWithTime;

    /**
     * 缓存前缀
     */
    const PREFIX = 'CH_';
    /**
     * 默认缓存时间 1小时
     */
    const CUSTOM_TTL = 1 * 60 * 60;

    /**
     * @var \Redis
     */
    protected $redis;

    public function __construct() {
        $this->redis = Redis::getRedis();
    }

    /**
     * @param string $key
     * @return string
     */
    private function getCacheKey(string $key) {
        return self::PREFIX . $key;
    }

    /**
     * @param string $key
     * @param null $default
     * @param null|int|DateInterval $ttl
     * @return false|mixed|string
     */
    public function get(string $key, $default = null, $ttl = self::CUSTOM_TTL) {
        $value = $this->redis->get($this->getCacheKey($key));
        if ($value === false) {
            if ($default != null) {
                if ($default instanceof \Closure) {
                    $value = $default();
                } else {
                    $value = $default;
                }
                if ($value !== null) {
                    $this->set($key, $value, $ttl);
                }
            }
        }
        return $value;
    }

    /**
     * @param string $key
     * @param string|int|mixed $value
     * @param null|int|DateInterval $ttl
     */
    public function set(string $key, $value, $ttl = null) {
        $seconds = $this->secondsUntil($ttl);
        $this->redis->set($this->getCacheKey($key), $value, $seconds > 0 ? $seconds : null);
    }

    /**
     * @param string $key
     * @return bool
     */
    public function delete($key) {
        return (bool) $this->redis->del($this->getCacheKey($key));
    }

    /**
     * @param $key
     * @return bool
     */
    public function has($key) {
        return (bool) $this->redis->exists($this->getCacheKey($key));
    }

}